/*------------------------------------------------------------------------/
/                                                                         /
/ Copyright                                                               /
/    IBM Confidential                                                     /         
/    OCO Source Materials                                                 /          
/    5739-B75 5739-B74 5765-C09 and Other Materials                       /         
/    (C) Copyright IBM Corp. 1997                                         /        
/    The source code for this program is not published or otherwise       /       
/    divested of its trade secrets, irrespective of what has been         /      
/    deposited with the U.S. Copyright Office.                            /     
/                                                                         /
/------------------------------------------------------------------------*/

#ifndef _DICTIONARY_H_
#define _DICTIONARY_H_

extern "C" {
#include <StubPreamble.h>
}

// Currently the dictionary is implemented using a linked list. This is certainly
// not the most elegant approach. In a final relase the dictionary implementation
// should be changed to a hash table.

struct _HandleDict {
  int64_t jobj;
  int64_t cobj;
  struct _HandleDict *nxt;
};

typedef struct _HandleDict * HandleDict;

// The class Dictionary has two public accessor functions EnterObjectPair and
// GetJavaObject. They are based on two private method which must be changed
// if the implemenation changes. The accessor function could remain untouched.
// All methods are declared static. Therefore the Dictionary class is never
// instantiated.
//
// The dictionary maintains items in the form
// [ address of Java instance , address of C++ instance ]
// which are entered whenever an object is created.
// The lookup method searches for already registered instances of C++
// objects and returns the pointer to the Java object.

class Dictionary {
public:
	// accessor method to enter an item in the dictionary
	static void EnterObjectPair(int64_t, int64_t);

        // accessor method to look up 
	static int64_t GetJavaObject(int64_t);

        // accessor method to remove an item
        static void RemoveEntry(int64_t);

private:
        // implemenatation specific part
	static void _EnterObjectPair(int64_t, int64_t);
	static int64_t _GetJavaObject(int64_t);
	static void _RemoveEntry(int64_t);

        // linked list
        static HandleDict dictionary;
};

#endif

